# -*- coding:utf-8 -*-
import os
from collections import OrderedDict
from ConfigParser import ConfigParser


_multi = lambda x: x.strip().split("\n")


class Parser(ConfigParser):
    def getdict(self, section, multiple_value_names=[], defaults={}):
        items = self.items(section)
        multi = lambda k, v: _multi(v) if k in multiple_value_names else v
        return OrderedDict((k, multi(k, v.format(**defaults)))
                           for k, v in items)


def make_task(section, base_dir, values):
    values = values.copy()
    recipefile = values.pop("recipefile", None)
    if recipefile:
        fabfile = recipefile
    # cookbooks.auth:setup_all -> cookbooks/auth.py
    filename, task_name = section.split(":")[:2]
    filename = filename.replace(".", "/")

    fabfile = "{0}/{1}.py".format(base_dir, filename)
    if values:
        arg_str = ",".join("{0}={1}".format(k, v) for k, v in values.items())
        task = "{0}:{1}".format(task_name, arg_str)
    else:
        task = task_name
    return fabfile, task


def make_command(fabfile, task, options):
    return "fab {options} -f {fabfile} {task}".format(
        options=options,
        fabfile=fabfile,
        task=task
    )


def fabcommands(filepath):
    with open(filepath, "r") as fp:
        config = Parser()
        config.readfp(fp)
        fabric = config.getdict("fabric", ["options", "tasks"])
        base_dir = fabric.get("base_dir")
        defaults = {"base_dir": base_dir}
        for section in fabric["tasks"]:
            values = config.getdict(section, defaults=defaults)
            fabfile, task = make_task(section, base_dir, values)
            command = make_command(fabfile, task, " ".join(fabric["options"]))
            yield command


def main():
    sample = os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        "sample.cfg"
    )
    for command in fabcommands(sample):
        print command


if __name__ == "__main__":
    main()
